package net.es.oscars.policy.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import net.es.oscars.database.HibernateUtil;
import net.es.oscars.database.Initializer;
import net.es.oscars.policy.database.*;

import org.hibernate.Session;

import jargs.gnu.CmdLineParser;

public class RuleUtil {
    
    private String dbname = "oscars_policy";

    public RuleUtil(){
        /* Init database */
        Initializer initializer = new Initializer();
        ArrayList<String> dbnames = new ArrayList<String>();
        dbnames.add(this.dbname);
        initializer.initDatabase(dbnames);
    }

    public void addSet(String setName) {
        Session sess =
            HibernateUtil.getSessionFactory(this.dbname).getCurrentSession();
        sess.beginTransaction();
        Scanner in = new Scanner(System.in);
        RuleSetDAO ruleSetDAO = new RuleSetDAO(this.dbname);
        RuleDAO ruleDAO = new RuleDAO(this.dbname);
        RuleSet ruleSet = ruleSetDAO.queryByParam("name", setName);
        boolean newSet = false;
        if(ruleSet == null){
            ruleSet = new RuleSet();
            ruleSet.setName(setName);
            newSet = true;
        }
        
        do{
            Rule rule = this.createRule(in);
            ruleDAO.create(rule);
            if(ruleSet.getRule() == null || ("RULE".equals(rule.getLType().getName()) && 
                    (rule.getLValue().equals(ruleSet.getRule().getId().toString()) ||
                     rule.getRValue().equals(ruleSet.getRule().getId().toString())))){
                ruleSet.setRule(rule);
            }
            
            String ans = "";
            while(!(ans.startsWith("n") || ans.startsWith("y"))){
                System.out.println("Do you want to add another rule?(y/n) ");
                ans = in.next().toLowerCase().trim();
            }
            if(ans.startsWith("n")){
                break;
            }
        }while(true);
        
        if(newSet){
            ruleSetDAO.create(ruleSet);
        }
        
        sess.getTransaction().commit();
    } 
    
    private Rule createRule(Scanner in) {
        Rule rule = new Rule();
        int n = 1;
        int index = -1;
        LTypeDAO lTypeDAO = new LTypeDAO(this.dbname);
        List<LType> lTypes = lTypeDAO.list();
        while(index < 0 || index >= lTypes.size()){
            for(LType lType : lTypes){
                System.out.println(n + ". " + lType.getName());
                n++;
            }
            System.out.print("Choose type of left operand: ");
            index = in.nextInt() - 1;
        }
        rule.setLType(lTypes.get(index));
        
        n = 1;
        index = -1;
        if("ACTION".equals(rule.getLType().getName())){
            rule.setLValue("*");
        }else if("RULE".equals(rule.getLType().getName())){
            rule.setLValue(this.chooseRule(in).getId().toString());
        }else{
            System.out.print("Set left operand value: ");
            rule.setLValue(in.next());
        }
        n = 1;
        index = -1;
        OperatorDAO opDAO = new OperatorDAO(this.dbname);
        List<Operator> ops = opDAO.list();
        while(index < 0 || index >= ops.size()){
            for(Operator op : ops){
                System.out.println(n + ". " + op.getName() + "(" + op.getRType().getName() + ")");
                n++;
            }
            System.out.print("Choose operator: ");
            index = in.nextInt() - 1;
        }
        rule.setOperator(ops.get(index));
        
        n = 1;
        index = -1;
        if("RULE".equals(rule.getOperator().getRType().getName())){
            rule.setRValue(this.chooseRule(in).getId().toString());
        }else{
            System.out.print("Set right operand value: ");
            rule.setRValue(in.next());
        }
        return rule;
    }
    
    public Rule chooseRule(Scanner in){
        RuleDAO ruleDAO = new RuleDAO(this.dbname);
        List<Rule> existingRules = ruleDAO.list();
        int m = 1;
        for(Rule existingRule : existingRules){
            String left = existingRule.getLValue();
            if("*".equals("left")){
                left = existingRule.getLType().getName();
            }else if("RULE".equals(existingRule.getLType().getName())){
                left = "RULE("+left+")";
            }
            String right = existingRule.getRValue();
            if("RULE".equals(existingRule.getOperator().getRType().getName())){
                right = "RULE("+right+")";
            }
            
            System.out.println(m + ". " + left + " " + 
                    existingRule.getOperator().getName() + " " + right + 
                    "("+existingRule.getId()+")");
            m++;
        }
        int choice = 0;
        while(choice < 1 || ((choice - 1) >= existingRules.size() )){
            System.out.print("Choose a rule: ");
            choice = in.nextInt();
        }
        
        return existingRules.get(choice-1);
    }

    public void addClass(String className, String setName) {
        Session sess =
            HibernateUtil.getSessionFactory(this.dbname).getCurrentSession();
        sess.beginTransaction();
        Scanner in = new Scanner(System.in);
        PolicyClassDAO policyClassDAO = new PolicyClassDAO(this.dbname);
        PolicyClassCriterionDAO policyClassCritDAO = new PolicyClassCriterionDAO(this.dbname);
        PolicyClassCriterionTypeDAO policyClassCritTypeDAO = new PolicyClassCriterionTypeDAO(this.dbname);
        RuleSetDAO ruleSetDAO = new RuleSetDAO(this.dbname);
        PolicyClass policyClass = new PolicyClass();
        policyClass.setName(className);
        
        int choice = 0;
        List<PolicyClass> policyClasses = policyClassDAO.list();
        while(choice < 1 || ((choice - 1) > policyClasses.size() )){
            System.out.println("1. None");
            int n =2;
            for(PolicyClass existingClass: policyClasses){
                System.out.println(n + ". " + existingClass.getName());
                n++;
            }
            System.out.print("Select parent class: ");
            choice = in.nextInt();
        }
        
        if(choice != 1){
            policyClass.setParent(policyClasses.get(choice-2));
        }
        
        //set criteria
        List<PolicyClassCriterion> criteria = new ArrayList<PolicyClassCriterion>();
        while(true){
            System.out.println();
            System.out.println("Add new criteria for belonging to class: ");
            PolicyClassCriterion policyCrit = new PolicyClassCriterion();
            List<PolicyClassCriterionType> types = policyClassCritTypeDAO.list();
            int typeChoice = -1;
            int n = 1;
            for(PolicyClassCriterionType type : types){
                System.out.println(n + ". " + type.getName());
                n++;
            }
            while(typeChoice < 0 || typeChoice > types.size()){
                System.out.print("Select criterion type: ");
                typeChoice = in.nextInt() - 1;
                in.nextLine().trim();
            }
            policyCrit.setType(types.get(typeChoice));
            System.out.print("Enter value that must be matched: ");
            String val = in.nextLine().trim();
            policyCrit.setValue(val);
            policyCrit.setPolicyClass(policyClass);
            criteria.add(policyCrit);
            
            String ans = "";
            while(!(ans.startsWith("n") || ans.startsWith("y"))){
                System.out.print("Would you like to add another criterion?[y/n]");
                ans = in.next().trim();
                ans = ans.toLowerCase();
            }
            if(ans.startsWith("n")){
                break;
            }
        }
        
        if(setName != null){
            policyClass.setRuleSet(ruleSetDAO.queryByParam("name", setName));
        }else{
            choice = 0;
            List<RuleSet> ruleSets = this.listRuleSets();
            while(choice < 1 || ((choice - 1) > ruleSets.size() )){
                System.out.print("Select a rule set: ");
                choice = in.nextInt();
            }
            policyClass.setRuleSet(ruleSets.get(choice - 1));
        }
        
        String ans = "";
        do{
            System.out.print("Allow or deny action if rule constraints met? [allow/deny]");
            ans = in.next().toLowerCase();
        }while(!(ans.equals("allow") || ans.equals("deny")));
        policyClass.setAllowOnMatch(ans.equals("allow"));
        
        policyClassDAO.create(policyClass);
        for(PolicyClassCriterion criterion : criteria){
            policyClassCritDAO.create(criterion);
        }
        sess.getTransaction().commit();
    }
    
    public List<RuleSet> listRuleSets(){
        RuleSetDAO ruleSetDAO = new RuleSetDAO(this.dbname);
        List<RuleSet> ruleSets = ruleSetDAO.list();
        int n = 1;
        for(RuleSet ruleSet : ruleSets){
            System.out.println(n + ". " + ruleSet.getName());
        }
        return ruleSets;
    }
    public void printUsage(){
        System.out.println("Usage: policy-rule [{-s,--set} set_name][{-c,--class} class_name]");
    }
    
    public static void main(String[] args){
        RuleUtil util = new RuleUtil();
        CmdLineParser parser = new CmdLineParser();
        
        CmdLineParser.Option addSet = parser.addStringOption('s', "set");
        CmdLineParser.Option addClass = parser.addStringOption('c', "class");
        
        try{
            parser.parse(args);
        }catch ( CmdLineParser.OptionException e ) {
            System.err.println(e.getMessage());
            util.printUsage();
            System.exit(2);
        }
        
        String setName = (String)parser.getOptionValue(addSet);
        String className = (String)parser.getOptionValue(addClass);
        if(setName != null){
            util.addSet(setName);
        }
        if(className != null){
            util.addClass(className, setName);
        }
    }
}
